%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Apply a rule set on data instances, and return the labels
% Instances that are not covered are labeled '-1' or minimum label minus 
% one if '-1' is a label (Code based on evaluateRuleset.m Antminer+ code)

function [label,minLabel] = applyRuleset(ruleset,data,variableType,anyValue,rulesetClassIdx)

%% Function arguments
% - ruleset contains the rules
% - data contains the instances that need to be labeled, with the rows the
% observations and the columns the attributes
% - variableType indicates the type of variable in a column: nominal (0) or
% ordinal (1)
% - anyValue is the value in the ruleset representing "any" (nominal atts)
% - rulesetClassIdx is an argument that indicates the column with the
% target variable in the ruleset

%% Set no label indicator
minLabel = min(ruleset(:,rulesetClassIdx));
if minLabel < -1
    minLabel = floor(minLabel-1);
else
    minLabel = -1;
end

%% Evaluate rule set accuracy on test set
nextRuleIndex = size(ruleset,1); % the number of rules in the ruleset
nbVariables = size(variableType,2); % the number of variables in the data set
nrInstances = size(data,1);
label = zeros(nrInstances,2);
for i = 1:nrInstances %cover all datapoints in the test set
    for j = 1:nextRuleIndex %try each rule in order
        condition = 1;
        idx = 1;
        for k = 1:nbVariables %check if the datapoint matches the rule
            if variableType(k) == 0 %nominal variable
                if ruleset(j,idx) ~= anyValue %take into account the 'any'-case
                    if ruleset(j,idx) ~= data(i,k) 
                        condition = 0;
                        break;
                    end
                end
                idx = idx + 1;
            end
            if variableType(k) == 1 %ordinal variable => lower and upper bound
                if ruleset(j,idx) > data(i,k) %lower bound restriction
                    condition = 0;
                    break;
                end
%                 if ruleset(j,idx+1) < data(i,k) %upper bound restriction
                if ruleset(j,idx+1) <= data(i,k) %upper bound restriction
                    condition = 0;
                    break
                end
                idx = idx + 2; %skip the 2 nominal variables of the ordinal variable
            end
        end
        if condition == 1 %datapoint matches the j-th rule in the decision list
            label(i,1) = ruleset(j,rulesetClassIdx);
            label(i,2) = 1;
            break;
        end
    end
end
if nnz(label(:,2)==0)
    disp('warning: instances not covered by ruleset!')
    label(label(:,2)==0,1) = minLabel;
end
label = label(:,1);

