%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% RULEM function

function [NP,WC,grid]  = calcNPWC(ruleset,constraints,varType,trainData)

rulesetClassIdx = size(ruleset,2); % the class label is assigned by the last column in the rule set
nrVar = size(constraints,2);
varIndex = zeros(1,nrVar);
index = 1;
anyValue = []; % init: if no nominal attributes
for i = 1:nrVar
    varIndex(1,i) = index;
    if varType(1,i) == 0
        anyValue = ruleset(end,index); % the anyValue indicator is the same for each nominal variable, and is the value of the nominal variables in the defaultRule
        index = index + 1;
    else
        index = index + 2;
    end
end

%% Initialize the n-dimensional grid, with n the number of attributes
% disp('            Initializing grid');
nrCuts = zeros(1,nrVar);
cutVals = cell(1,nrVar);
nrCellsVar = zeros(1,nrVar);
for i = 1:nrVar
    if varType(1,i) == 0
        cutVals{1,i} = unique(ruleset(:,varIndex(1,i)));
        cutVals{1,i} = setdiff(cutVals{1,i},anyValue); % anyValue is not a separate cut of the grid for a nominal attribute
        if isempty(cutVals{1,i}) % then nominal attribute always takes value = anyValue
            cutVals{1,i}= anyValue;
        end
        nrCellsVar(1,i) = size(cutVals{1,i},1);
    elseif varType(1,i) == 1
        cutVals{1,i} = unique([ruleset(:,varIndex(1,i));ruleset(:,varIndex(1,i)+1)]);
        nrCellsVar(1,i) = size(cutVals{1,i},1)-1;
    end
    nrCuts(1,i) = size(cutVals{1,i},1);
end
cuts = zeros(max(nrCuts),nrVar);
for i = 1:nrVar
    cuts(1:nrCuts(1,i),i) =  cutVals{1,i}(:,1);
end
grid = zeros(nrCellsVar);
% Label each cell in the grid
nrCells = prod(nrCellsVar);
indices = ind2sub2(nrCellsVar,(1:+1:nrCells));
middleCell = zeros(1,nrVar);
for i = 1:nrCells
    for j = 1:nrVar % compose center of the cell to find label
        if varType(1,j) == 0
            middleCell(1,j) = cuts(indices(i,j),j);
        else
            middleCell(1,j) = (cuts(indices(i,j),j) + cuts(indices(i,j)+1,j))/2;
        end
    end
    grid(sub2ind2(size(grid),num2cell(indices(i,:)))) = applyRuleset(ruleset,middleCell,varType,anyValue,rulesetClassIdx);
end

%% Calculate NP, WC and compose NPWC
NP = calculateGridNP(grid,nrCellsVar,constraints);
WC = calculateGridWC(grid,nrCellsVar,constraints,varType,anyValue,trainData,cuts);
