%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% This function calculates the gridNP - RULEM

function [NP gridNP gridN gridP totalN totalP meanN] = calculateGridNP(grid,nrCellsVar,constraints)

%% Initialize parameters
nrVar = size(constraints,2);
varIDs = 1:+1:nrVar;
constrainedVarIDs = find(abs(constraints) == 1);
nrVarConstrained = size(constrainedVarIDs,2);
gridN = zeros(nrCellsVar);
gridP = zeros(nrCellsVar);

%% Calculate gridNP
for i = 1:nrVarConstrained % for every constrained variable
    redVarIDs = setdiff(varIDs,constrainedVarIDs(1,i)); % all varIDs except the current constrained var
    nrCellsRed = prod(nrCellsVar(1,redVarIDs)); % the number of cells in the reduced space
    indicesRed = ind2sub2(nrCellsVar(1,redVarIDs),(1:+1:nrCellsRed)); % reduce the grid - remove constrained dimension
    for j = 1:size(indicesRed,1) % for every cell in the reduced attribute space
        classOrder = zeros(1,nrCellsVar(1,constrainedVarIDs(1,i))); % init
        indices = zeros(nrCellsVar(1,constrainedVarIDs(1,i)),nrVar); % init
        for k = 1:nrCellsVar(1,constrainedVarIDs(1,i)) % for every cell in the constrained dimension
            indices(k,redVarIDs) = indicesRed(j,:);
            indices(k,constrainedVarIDs(1,i)) = k;
            classOrder(1,k) = grid(sub2ind2(size(grid),num2cell(indices(k,:))));
            if constraints(1,constrainedVarIDs(1,i)) == 1 % positive constraint
                for l = 1:k-1
                    if classOrder(1,k) >= classOrder(1,l) % no violation
                        gridP(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridP(sub2ind2(size(grid),num2cell(indices(k,:)))) + 1;
                        gridP(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridP(sub2ind2(size(grid),num2cell(indices(l,:)))) + 1;
                    else % violation
                        gridN(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridN(sub2ind2(size(grid),num2cell(indices(k,:)))) + 1;
                        gridN(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridN(sub2ind2(size(grid),num2cell(indices(l,:)))) + 1;
                    end
                end
            else % negative constraint
                for l = 1:k-1
                    if classOrder(1,k) <= classOrder(1,l) % no violation
                        gridP(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridP(sub2ind2(size(grid),num2cell(indices(k,:)))) + 1;
                        gridP(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridP(sub2ind2(size(grid),num2cell(indices(l,:)))) + 1;
                    else % violation
                        gridN(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridN(sub2ind2(size(grid),num2cell(indices(k,:)))) + 1;
                        gridN(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridN(sub2ind2(size(grid),num2cell(indices(l,:)))) + 1;
                    end
                end
            end
        end
    end
end

%% Calculate output arguments
% Current calculation of NP and gridNP
meanN = totalSum(gridN)/numel(gridN);
maxN = totalMax(gridN);
if maxN > 0
    gridN = gridN./maxN;
end
maxP = totalMax(gridP);
if maxP > 0 
    gridP = gridP./maxP;
end
totalN = totalSum(gridN);
totalP = totalSum(gridP);
NP = totalN;
gridNP = gridN;

% Previous calculation of NP and gridNP 
% totalN = totalSum(gridN);
% totalP = totalSum(gridP);
% 1:
% % To avoid divide by zero errors - and since
% % ranking N or N/P is equivalent: NP = N instead of N/P
% NP = totalN;
% gridNP = gridN;
% 2:
% NP = totalN/totalP;
% gridNP = gridN./gridP; 
