%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% This function calculates the gridWC - RUMII
% The function assumes that the class variable in the data is in the last
% column
% WC: wrong/correct

function [WC gridWC gridW gridC totalW totalC] = calculateGridWC(grid,nrCellsVar,constraints,varType,anyValue,trainData,cuts)

%% Initialize parameters
nrVar = size(constraints,2);
varIDs = 1:+1:nrVar;
constrainedVarIDs = find(abs(constraints) == 1);
nrVarConstrained = size(constrainedVarIDs,2);
varIndex = zeros(1,nrVar);
index = 1;
for i = 1:nrVar
    varIndex(1,i) = index;
    if varType(1,i) == 0
        index = index + 1;
    else
        index = index + 2;
    end
end
if varType(1,end) == 1
    ruleSize = varIndex(1,end)+2;
else
    ruleSize = varIndex(1,end)+1;
end
varIndices = 1:nrVar;
varOrd = varIndices(varType==1);
gridW = zeros(nrCellsVar);
gridC = zeros(nrCellsVar);

%% Evaluate cells in grid, resulting in gridCC - correctly classified 
%% instances in each cell of the grid
nrCells = prod(nrCellsVar);
indices = ind2sub2(nrCellsVar,(1:+1:nrCells));
gridCC = zeros(nrCellsVar);
for i = 1:nrCells
    cellRule = zeros(1,ruleSize);
    cellRule(1,varIndex) = cuts(sub2ind(size(cuts),indices(i,:),varIndices));
    cellRuleVals = cuts(sub2ind(size(cuts),(indices(i,:)+1),varIndices));
    cellRule(1,varIndex(1,varOrd)+1) = cellRuleVals(1,varType==1);
    cellRule(1,end) = grid(sub2ind2(size(grid),num2cell(indices(i,:))));
    [~,gridCC(sub2ind2(size(grid),num2cell(indices(i,:))))] = evalRuleset(cellRule,trainData,varType,anyValue,size(trainData,2),ruleSize,1);
end  
   
%% Calculate gridWC
for i = 1:nrVarConstrained
    redVarIDs = setdiff(varIDs,constrainedVarIDs(1,i));
    nrCellsRed = prod(nrCellsVar(1,redVarIDs)); 
    indicesRed = ind2sub2(nrCellsVar(1,redVarIDs),(1:+1:nrCellsRed));
    for j = 1:size(indicesRed,1)
        classOrder = zeros(1,nrCellsVar(1,constrainedVarIDs(1,i)));
        indices = zeros(nrCellsVar(1,constrainedVarIDs(1,i)),nrVar);
        for k = 1:nrCellsVar(1,constrainedVarIDs(1,i))
            indices(k,redVarIDs) = indicesRed(j,:);
            indices(k,constrainedVarIDs(1,i)) = k;
            classOrder(1,k) = grid(sub2ind2(size(grid),num2cell(indices(k,:))));
            if constraints(1,constrainedVarIDs(1,i)) == 1 % positive constraint
                for l = 1:k-1
                    if classOrder(1,k) >= classOrder(1,l) % no violation
                        gridC(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridC(sub2ind2(size(grid),num2cell(indices(k,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(l,:))));
                        gridC(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridC(sub2ind2(size(grid),num2cell(indices(l,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(k,:))));
                    else % violation
                        gridW(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridW(sub2ind2(size(grid),num2cell(indices(k,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(l,:))));
                        gridW(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridW(sub2ind2(size(grid),num2cell(indices(l,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(k,:))));
                    end
                end
            else % negative constraint
                for l = 1:k-1
                    if classOrder(1,k) <= classOrder(1,l) % no violation
                        gridC(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridC(sub2ind2(size(grid),num2cell(indices(k,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(l,:))));
                        gridC(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridC(sub2ind2(size(grid),num2cell(indices(l,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(k,:))));
                    else % violation
                        gridW(sub2ind2(size(grid),num2cell(indices(k,:)))) = gridW(sub2ind2(size(grid),num2cell(indices(k,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(l,:))));
                        gridW(sub2ind2(size(grid),num2cell(indices(l,:)))) = gridW(sub2ind2(size(grid),num2cell(indices(l,:)))) + gridCC(sub2ind2(size(grid),num2cell(indices(k,:))));
                    end
                end
            end
        end
    end
end

%% Calculate output arguments
% Current calculation of WC and gridWC
maxW = totalMax(gridW);
if maxW > 0
    gridW = gridW./maxW;
end
maxC = totalMax(gridC);
if maxC > 0
    gridC = gridC./maxC;
end
totalW = totalSum(gridW);
totalC = totalSum(gridC);
WC = totalW;
gridWC = gridW;

% Previous calculation of WC and gridWC
% totalW = totalSum(gridW);
% totalC = totalSum(gridC);
% 1: 
% WC = totalW/totalC
% gridWC = gridW./gridC
% 2:
% totalT = totalW + totalC;
% WC = totalW/totalT;
% % gridT elements can be 0 leading to NaN in gridWC; resolve by setting 
% % gridT(gridT==0) = gridT(gridT==0) + 1 since gridW(gridT==0) == 0 this
% % will result in gridWC(gridT==0) == 0
% gridT = gridC + gridW;
% gridT(gridT==0) = 1;
% gridWC = gridW./gridT;

