%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% This function calculates the justifiability of a rule set, based on the
% gridNP/gridC (C-scores of elementary grid) calculated using function
% "calculateGridNP". The fraction of the attribute space occupied by cells
% with NP score > 0 results in the RUMS justifiability measure, while the 
% RUMF measure is calculated as the fraction of the data that lies within
% this fraction of the attribute space

function [RULEMS RULEMF] = calculateJustifiability(gridC,cuts,nrCellsVar,...
                           defaultRule,data,varType)
%% Initialize
nrVar = numel(varType);
varIndex = zeros(1,nrVar);
index = 1;
anyValue = []; % init: if no nominal attributes
for i = 1:nrVar
    varIndex(1,i) = index;
    if varType(1,i) == 0
        anyValue = defaultRule(1,index); % the anyValue indicator is the same for each nominal variable, and is the value of the nominal variables in the defaultRule
        index = index + 1;
    else
        index = index + 2;
    end
end

%% Find cells different from zero, and describe with rules
indices = find(gridC);
if ~isempty(indices)
    rulesAdd = ind2sub2(nrCellsVar,indices);
    nrRulesAdd = size(rulesAdd,1);
    rulesetAdd = zeros(nrRulesAdd,numel(defaultRule)-1); %-1: no class label to be assigned
    for j = 1:nrVar
        rulesetAdd(:,varIndex(1,j)) = cuts(rulesAdd(:,j),j);
        if varType(1,j) == 1
            rulesetAdd(:,varIndex(1,j)+1) = cuts(rulesAdd(:,j)+1,j);
        end
    end
    % Calculation of RUMS and RUMF
    RULEMS = 1;
    nrValues = zeros(1,size(varType,2));
    for i = 1:size(varType,2)
        if varType(1,i) == 0
            nrValues(1,i) = numel(unique(data(:,i)));
        end
    end
    for i = 1:size(rulesetAdd,1)
        RULEMS = RULEMS - ruleSpace(rulesetAdd(i,:),defaultRule,varType,nrValues);
    end
    RULEMF = 1 - (size(coveredInstances(rulesetAdd,data,varType,anyValue),1))/size(data,1);
else
    RULEMS = 1;
    RULEMF = 1;
end
