%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Check which instances in a data set are covered by a rule

function coveredInstancesIDs = coveredInstances(rule,data,valueTypes,anyValue)

% rule contains the rules
% data contains the instances, with the rows the observations and the
% columns the attributes
% valueTypes indicates the type of variable in a column: nominal (0) or
% ordinal (1)
% anyValue is the value in the rule representing "any"

%% Evaluate rule set accuracy on test set
nextRuleIndex = size(rule,1); % the number of rules in the rule
nbVariables = size(valueTypes,2); % the number of variables in the data set
nbTest = size(data,1);
coveredInstancesIDs = [];
for i = 1:nbTest %cover all datapoints in the test set
    for j = 1:nextRuleIndex %try each rule in order
        condition = 1;
        idx = 1;
        for k = 1:nbVariables %check if the datapoint matches the rule
            if valueTypes(k) == 0 %nominal variable
                if rule(j,idx) ~= anyValue %take into account the 'any'-case
                    if rule(j,idx) ~= data(i,k)
                        condition = 0;
                        break;
                    end
                end
                idx = idx + 1;
            end
            if valueTypes(k) == 1 %ordinal variable => lower and upper bound
                if rule(j,idx) > data(i,k) %lower bound restriction
                    condition = 0;
                    break;
                end
                if rule(j,idx+1) <= data(i,k) %upper bound restriction
%                 if rule(j,idx+1) < data(i,k) %upper bound restriction
                    condition = 0;
                    break
                end
                idx = idx + 2; %skip the 2 nominal variables of the ordinal variable
            end
        end
        if condition == 1 %datapoint matches the j-th rule in the decision list
            coveredInstancesIDs = [coveredInstancesIDs;i];
            break;
        end
    end
end
