%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Function to calculate the degree monotonicity of a data set as formulated
% in Daniels & Velikova (2010)

% class label in data should be in the last column, instances are rows in
% data, constraints is vector of length = nr. of attributes, with 1
% indicating that positive constraint is imposed on attribute, -1
% indicating that a negative constraint is imposed on attribute, and zero
% indicating that no constraint is imposed.

function [deg monPairs comPairs] = degMon(data,constraints)

numInst = size(data,1);
monPairs = 0;
comPairs = 0;
negInd = find(constraints == -1);
if ~isempty(negInd)
    data(:,negInd) = data(:,negInd)*-1;
end

% for i = 1:numInst
%     for j = i+1:numInst
%         if ~nnz((data(i,1:end-1) >= data(j,1:end-1))==0)
%             comPairs = comPairs + 1;
%             if data(i,end) >= data(j,end)
%                 monPairs = monPairs + 1;
%             end
%         end
%     end
% end

for i = 1:numInst
    ind = sum((data(i+1:end,1:end-1) < repmat(data(i,1:end-1),numInst-i,1)),2);
    indComp = find(ind==0);
    comPairs = comPairs + numel(indComp);
    indMon = indComp(data(i+indComp,end) >= data(i,end));
    monPairs = monPairs + numel(indMon);
end

deg = monPairs/comPairs;

