%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Derive ruleset from J48 model created by Weka for RULEM postprocessing

function [ruleset defaultRule anyValue] = getRulesetJ48(modelString,variableType,mdata,featureNames,adjustmentValue)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%WARNINGS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% WARNING:
% 1) if a '=' '<' or '>' operator is present in a name of the 
%    variables, an error will occur!
% 2) if the name of a variable is a part of the name of another variable,
%    then this will result in a wrong rule set!
% 3) if empty tree does not function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ERRORS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%TO-DO%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TO DO: in simpleCart (not in J48?) as well the != sign is possible:
%        currently not included in reading mechanism
% TO DO: checks whether mdata is in the right format (numeric, etc.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%TO-DO%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The last attribute should be the class attribute, with name "class"
% The data mdata is supposed to be fully numeric (no categorical attributes
% with strings as possible labels)
% class labels should be starting from zero and increasing - this can be
% achieved by reading an arff file into matlab, and writing it back to 
% .arff file

%% Initialize parameters
minData = min(mdata);
maxData = max(mdata);
anyValue = max(maxData)+1;
nrVariables = size(featureNames,2)-1;
varIndex = zeros(1,nrVariables);
varIndex(1,1) = 1; % varIndex indicates the start-antecedent for each variable
index = 1;
for i = 2:nrVariables
    if variableType(1,i-1) == 0
        index = index + 1;
    else
        index = index + 2;
    end
    varIndex(1,i) = index;
end
nrRules = strfind(modelString,'(');
nrRules = size(nrRules,2);

%% Initialize ruleset
ruleset = zeros(nrRules,1);
for i = nrVariables:-1:1
    if variableType(1,i) == 1
        values = zeros(nrRules,2);
        values(:,1) = minData(1,i);
        values(:,2) = maxData(1,i)+adjustmentValue;
        ruleset = [values ruleset];
    elseif variableType(1,i) == 0
        values = zeros(nrRules,1);
        values(:,1) = anyValue;
        ruleset = [values ruleset];
    end
end
defaultRule = ruleset(1,1:end-1);

%% Read tree & convert to rules
s = textscan(modelString,'%s','delimiter','\n');
rules = cell(nrRules,1);
ruleIndex = 1;
for i = 4:size(s{1,1},1)-4
    k = strfind(s{1,1}{i,1},'|');
    endleaf = strfind(s{1,1}{i,1},'(');
    nrNodes = size(k,2);
    if nrNodes ==0 % startnode
        rule = {s{1,1}{i,1}};
        previousNrNodes = 0;
        if ~isempty(endleaf)
            rules{ruleIndex,1} = rule;
            ruleIndex = ruleIndex+1;
        end
    elseif nrNodes > previousNrNodes
        rule = {rule{1,1:end},s{1,1}{i,1}(1,k(1,nrNodes)+4:end)};
        previousNrNodes = nrNodes;
        if ~isempty(endleaf)
            rules{ruleIndex,1} = rule;
            ruleIndex = ruleIndex+1;
        end
    else
        previousNrNodes = nrNodes;
        rule = {rule{1,1:previousNrNodes},s{1,1}{i,1}(1,k(1,nrNodes)+4:end)};
        if ~isempty(endleaf)
            rules{ruleIndex,1} = rule;
            ruleIndex = ruleIndex+1;
        end
    end
end
for i = 1:nrRules
    rule = '';
    for j = 1:size(rules{i,1},2)-1
        rule = strcat(rule,'(',rules{i,1}{1,j},')', ' and ');
    end
    rule = strcat(rule,'(',rules{i,1}{1,end});
    rules{i,1} = rule;
end

for i = 1:nrRules
    rule = strrep(rules{i,1},': ', ') => class=');
    rules{i,1} = rule;
end

%% CONVERT RULES
for i = 1:nrRules
    for j = 1:size(featureNames,2)-1 % we assume last feature is class
        k = strfind(rules{i,1},featureNames{1,j}); % WARNING - if one variable name is part of other var name!
        for l = 1:size(k,2)
            m = strfind(rules{i,1}(1,k(1,l):end),'=');
            n = strfind(rules{i,1}(1,k(1,l):end),'<');
            o = strfind(rules{i,1}(1,k(1,l):end),'>');
            if ~isempty(n) && n(1,1) < m(1,1)-1 
                if isempty(o) || o(1,1) > n(1,1) % then a '<' sign
                    value = sscanf(rules{i,1}(1,k(1,l)-1+n(1,1)+2:end),'%f',1);
%                     value = value-adjustmentValue;
                    if ruleset(i,varIndex(1,j)+1) > value || ruleset(i,varIndex(1,j)+1)  == -1
                        ruleset(i,varIndex(1,j)+1) = value;
                    end
                end
            elseif ~isempty(0) && o(1,1) < m(1,1)-1 % then a '>' sign
                value = sscanf(rules{i,1}(1,k(1,l)-1+o(1,1)+2:end),'%f',1);
                value = value+adjustmentValue;
                if ruleset(i,varIndex(1,j)) < value
                    ruleset(i,varIndex(1,j)) = value;
                end                
            else % '=', '<=', '>='
                % ordinal variable
                if strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),'<') == 1
                    value = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2:end),'%f',1);
                    value = value+adjustmentValue;
                    if ruleset(i,varIndex(1,j)+1) > value || ruleset(i,varIndex(1,j)+1)  == -1
                        ruleset(i,varIndex(1,j)+1) = value;
                    end
                elseif strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),'>') == 1
                    value = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2:end),'%f',1);
                    if ruleset(i,varIndex(1,j)) < value
                        ruleset(i,varIndex(1,j)) = value;
                    end
                % nominal variable
                elseif strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),' ') == 1 && strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)+1),' ') == 1
                    ruleset(i,varIndex(1,j)) = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2),'%f',1);
                else
                    disp('Error reading rule set..')
                end
            end
        end
    end
    %set class label rule
    m = strfind(rules{i,1},'=');
    ruleset(i,end) = sscanf(rules{i,1}(1,m(1,end)+1:end),'%f',1);
end


