%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Derive ruleset from Ripper model built by Weka for RULEM postprocessing

function [ruleset anyValue] = getRulesetRipper(modelString,variableType,mdata,featureNames)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%TO-DO%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TO DO: checks whether mdata is in the right format (numeric, etc.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%TO-DO%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The rule set format derived from the modelString containing the rule set
% in "original" weka format is identical to the rule set format described
% by Bart Minnaert in AntMiner+
% The last attribute should be the class attribute, with name "class"
% The data mdata is supposed to be fully numeric (no categorical attributes
% with strings as possible labels) - class labels starting
% from zero and increasing - this can be achieved by reading an arff file
% into matlab, and writing it back to .arff file

%% MANUAL INPUT - iris.arff does not have numeric class labels!
% wekaTrainingObj = loadARFF('iris.arff');
% wekaClassifier = trainWekaClassifier(wekaTrainingObj,'rules.JRip');
% modelString = char(wekaClassifier.toString);
% [mdata,featureNames] = weka2matlab(wekaTrainingObj);
% variableType = [1 1 1 1];

%% INITIALIZE PARAMETERS
minData = min(mdata);
maxData = max(mdata);
anyValue = max(maxData)+1;  % anyValue is set one higher than the maximum value over all attribute values (including numeric/ordinal attributes)
nrVariables = size(featureNames,2)-1;
varIndex = zeros(1,nrVariables);
varIndex(1,1) = 1; % varIndex indicates the start-antecedent for each variable
index = 1;
for i = 2:nrVariables
    if variableType(1,i-1) == 0
        index = index + 1;
    else
        index = index + 2;
    end
    varIndex(1,i) = index;
end

%% DERIVE RULES in raw format
k = strfind(modelString, 'class');
nrRules = size(k,2);
rules = cell(nrRules,1);
classes = rules;
start = strfind(modelString,'(');
rules{1,1} = modelString(1,start:k(1,1)-4);
for i = 2:nrRules
    rules{i,1} = modelString(1,k(i-1):k(i));
    l = strfind(rules{i},'=');
    m = strfind(rules{i},'(');
    n = strfind(rules{i},')');
    classes{i-1,1} = rules{i,1}(1,l(1,1)+1:m(1,1)-1);
    rules{i,1} = rules{i,1}(n(1,1)+1:end-4);
end
rules{nrRules,1} = modelString(1,k(nrRules):end);
l = strfind(rules{nrRules,1},'=');
m = strfind(rules{nrRules,1},'(');
classes{nrRules,1} = rules{nrRules,1}(1,l(1,1)+1:m(1,1)-1);
rules{nrRules,1} = '';

%% Initialize ruleset: all rules are set equal to defaultrule
ruleset = zeros(nrRules,1);
for i = nrVariables:-1:1
    if variableType(1,i) == 0
        values = zeros(nrRules,1);
        values(:,1) = anyValue;
        ruleset = [values ruleset];
    elseif variableType(1,i) == 1
        values = zeros(nrRules,2);
        values(:,1) = minData(1,i);
        values(:,2) = maxData(1,i);
        ruleset = [values ruleset];
    end
end

%% PROCESS RULES 
% Ripper only results in '=', '>=' or '<=' operators (no '!=', '<' or '>')
for i = 1:size(rules,1)-1 % -1: defaultrule is not read
    for j = 1:size(featureNames,2)-1 % we assume last feature is class
        k = strfind(rules{i,1},featureNames{1,j});
        for l = 1:size(k,2)
            m = strfind(rules{i,1}(1,k(1,l):end),'=');
            % ordinal variable
            if strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),'<') == 1
                value = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2:end),'%f',1);
                if ruleset(i,varIndex(1,j)+1) > value
                    ruleset(i,varIndex(1,j)+1) = value;
                end
            elseif strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),'>') == 1
                value = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2:end),'%f',1);
                if ruleset(i,varIndex(1,j)) < value
                    ruleset(i,varIndex(1,j)) = value;
                end
            % nominal variable
            elseif strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)-1),' ') == 1 && strcmp(rules{i,1}(1,k(1,l)-1+m(1,1)+1),' ') == 1
                ruleset(i,varIndex(1,j)) = sscanf(rules{i,1}(1,k(1,l)-1+m(1,1)+2),'%f',1);
            else
                disp('Error reading rule set..')
            end
        end
    end
end

%% FILL OUT CLASS LABELS
for i = 0:maxData(1,end)
    for j = 1:nrRules
        if strfind(classes{j,1},num2str(i))
            ruleset(j,end) = i;
        end
    end
end
