% function fn_overSample(orFilename,newFilename,times)
% Thomas Verbraken - 03/30/2010 - Katholieke Universiteit Leuven
%
% The function reads an arff file and applies oversampling to the dataset.
% The user can give an array times, which contains in the first column the 
% values the target variable can take, and in the second column the number 
% of times these instances should be duplicated in the dataset. E.g. if 
% 2.5 is specified, the function will copy the current data twice and will 
% apply a random sample to come up with the last .5. The array times is 
% optional, if discarded, the function will strive for an even class 
% distribution

function fn_overSample(orFilename,newFilename,varargin)

wekaOBJ = loadARFF(orFilename);
[allData,featureNames,~,stringVals,~] = weka2matlab(wekaOBJ,[]);

if isempty(varargin)
    var_class = unique(allData(:,end));
    times = zeros(length(var_class),2);
    for i=1:length(var_class)
        times(i,1) = var_class(i);
        times(i,2) = sum(allData(:,end)==var_class(i));
    end
    times(:,2) = times(:,2)/max(times(:,2));
    times(:,2) = 1./times(:,2);
else
    times = varargin{1};
end
xtra_inst = [];
for i=1:size(times,1)
    idx = find(allData(:,end)==times(i,1));
    tms = 0;
    while times(i,2)-tms > 2
        xtra_inst = [xtra_inst;allData(idx,:)];
        tms = tms + 1;
    end
    frac = times(i,2)-tms-1;
    k = round(frac * length(idx));
    xtra_inst = [xtra_inst;allData(idx(randsample(1:length(idx),k)),:)];
end
allData = [allData;xtra_inst];
% save output
wekaOBJ = matlab2weka('SplitData_missValues_clustered_discret_oversample', featureNames, allData);
idx_arff = strfind(newFilename,'.arff');
saveARFF([newFilename(1:idx_arff-1) '_TEMP.arff'],wekaOBJ);
% find nom type
nom_type = [];
for i=1:length(stringVals)
    if ~isempty(stringVals{i})
        nom_type = [nom_type i];
    end
end
fn_convert_num2nom([newFilename(1:idx_arff-1) '_TEMP.arff'],newFilename,nom_type)


        
    