% function fn_replaceMissing_fromARFF(orFilename,newFilename,varargin)
% Thomas Verbraken - 03/30/2010 - Katholieke Universiteit Leuven
%
% This function replaces missing values in a given training and test set.
% Optional input arguments are mode_imp, median_imp and mean_imp. Whether
% all three should be defined or none, and they contain the attributes
% to which the respective imputation should be applied. If left blank, mode
% imputation is applied for all nominal attributes, and median imputation
% for all numeric attributes. This function takes an arff file as input 
% produces an arff file as output


function fn_replaceMissing_fromARFF(orFilename,newFilename,varargin)

if isempty(varargin)
    warning('No specification given: median imputation for all numeric attributes; mode imputation for all nominal attributes')
else if length(varargin) == 3
        mode_imp = varargin{1};
        median_imp = varargin{2};
        mean_imp = varargin{3};
    else
        error('Incorrect numer of arguments')
    end
end
        
%% Connect to weka and load file
wekaOBJ = loadARFF(orFilename);
[allData,featureNames,~,stringVals,~] = weka2matlab(wekaOBJ,[]);

%% check which attributes are nominal/numeric
nom_type = [];
for i=1:length(stringVals)
    if ~isempty(stringVals{i})
        nom_type = [nom_type i];
    end
end
num_type = setdiff(1:size(allData,2),nom_type);
% set the default imputation values
if isempty(varargin)
    median_imp = num_type;
    mode_imp = nom_type;
    mean_imp = [];
end    

%% imputation
% mode imputation
for j = 1:length(mode_imp)
    attr_i = mode_imp(j);
    idx_miss = find(isnan(allData(:,attr_i)));
    idx_notmiss = setdiff(1:size(allData,1),idx_miss);
    allData(idx_miss,attr_i) = mode(allData(idx_notmiss,attr_i),1);
end
if isempty(mode_imp)
    warning('No mode imputation applied')
end
        
% median imputation
for j = 1:length(median_imp)
    attr_i = median_imp(j);
    idx_miss = find(isnan(allData(:,attr_i)));
    idx_notmiss = setdiff(1:size(allData,1),idx_miss);
    allData(idx_miss,attr_i) = mode(allData(idx_notmiss,attr_i),1);
end
if isempty(median_imp)
    warning('No median imputation applied')
end

% mean imputation
for j = 1:length(mean_imp)
    attr_i = mean_imp(j);
    idx_miss = find(isnan(allData(:,attr_i)));
    idx_notmiss = setdiff(1:size(allData_trn,1),idx_miss);
    allData(idx_miss,attr_i) = mode(allData_trn(idx_notmiss,attr_i),1);
end
if isempty(mean_imp)
    warning('No mean imputation applied')
end

%% creating temporary arff file and saving to definite one
newWekaOBJ = matlab2weka('newmdata', featureNames, allData);
saveARFF([newFilename(1:idx_arff-1) '_tst_TEMP.arff'],newWekaOBJ);

% converting the arff file from numeric to nominal values
fn_convert_num2nom([newFilename(1:idx_arff-1) '_tst_TEMP.arff'],newFilename,nom_type);
