%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% Merge neighboring additional rules - RULEM

function rulesetAdd = mergeCells(rulesAdd,rulesetAdd,cuts,varType)

%% Initialize parameters
nrVar = size(varType,2);
varIndices = 1:nrVar;
classIdx = size(rulesAdd,2);
varIndex = zeros(1,nrVar);
index = 1;
for i = 1:nrVar
    varIndex(1,i) = index;
    if varType(1,i) == 0
        index = index + 1;
    else
        index = index + 2;
    end
end
nrRulesAdd = size(rulesAdd,1);
ruleIndicesOr = 1:nrRulesAdd;

%% Minimize nr of additional rules by merging neighboring cell-rules
merge = 1;
rulesetAddFinal = [];
varOrd = varIndices(varType==1);
nrVarOrd = numel(varOrd);
while merge
    % find maxSeq, longest sequence (number) of neighboring cells
    maxSeq = [];
    nrRulesAdd = size(rulesAdd,1);
    ruleIndices = 1:nrRulesAdd;
    for i = 1:nrRulesAdd
        diff = rulesAdd - repmat(rulesAdd(i,:),nrRulesAdd,1);
        for k = 1:nrVarOrd
            varIndicesK = setdiff([varIndices classIdx],varOrd(k)); % select all variables except the k-th ordinal variable AND CLASS LABEL!!
            ruleIndicesK = ruleIndices(sum(diff(:,varIndicesK)~=0,2)==0); % select the rules which are identical except for the k-th ordinal variable 
            seq = SplitVec2(diff(ruleIndicesK,varOrd(k)),'consecutive','index'); % select the rules that have consecutive values for variable k and that are identical in all other variables (ruleIndicesK)
            if numel(seq) > 1 && numel(seq) > numel(maxSeq)-1
                maxSeq = [ruleIndicesK(seq) k]; % we keep k to indicate for which variable..
            end
        end
    end
    if isempty(maxSeq)
        rulesetAdd = [rulesetAddFinal;rulesetAdd];
        merge = 0;
    else % merge maxSeq and remove from rulesAdd
        disp(['                comb: ' num2str(ruleIndicesOr(maxSeq(1:end-1)))]);
        indices = setdiff((1:nrRulesAdd),maxSeq(1:end-1));
        mergedRule = rulesetAdd(maxSeq(1,1),:);
        mergedRule(1,varIndex(1,maxSeq(1,end))) = cuts(min(rulesAdd(maxSeq(1,1:end-1),maxSeq(1,end))),maxSeq(1,end));
        mergedRule(1,varIndex(1,maxSeq(1,end))+1) = cuts(max(rulesAdd(maxSeq(1,1:end-1),maxSeq(1,end)))+1,maxSeq(1,end));
        rulesetAddFinal = [rulesetAddFinal;mergedRule];
        rulesetAdd = rulesetAdd(indices,:);
        rulesAdd = rulesAdd(indices,:);
        ruleIndicesOr = ruleIndicesOr(indices);
    end
end