%% Author: Wouter Verbeke
% Data Analytics Laboratory
% Department of Business Technology and Operations 
% Faculty of Economic and Social Sciences and Solvay Business School
% Vrije Universiteit Brussel, Pleinlaan 2, B-1050 Brussel, Belgium

%% Documentation
% This function calculates the fraction of the attribute space that is 
% occupied by a rule
% rule, variableType, and defaultRule are standard following RUM convention
% defaultRule covers the entire attribute space
% nrValues contains for nominal attributes the number of possible values
% (not maxValue), for ordinal attributes nrValues nrValues is not used

function fraction = ruleSpace(rule,defaultRule,variableType,nrValues)

fraction = 1;
varIndex = 1;
for i = 1:size(variableType,2) % for loop over the attributes
    if variableType(1,i) == 0  % nominal variable
        if rule(1,varIndex) < defaultRule(1,varIndex)
            fraction = fraction*(1/nrValues(1,i)); % not one over defaultRule(1,varIndex) 
            % since defaultRule contains maxValue, which is not the number of possible values 
            % a categorical attribute can take
        end
        varIndex = varIndex + 1;
    else
        fraction = fraction*(rule(1,varIndex+1) - rule(1,varIndex))/(defaultRule(1,varIndex+1) - defaultRule(1,varIndex));
        varIndex = varIndex + 2;
    end
end

